#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Simple plotter using output of snlc 
from snfit package version 2.4.1 available there :
http://supernovae.in2p3.fr/salt
"""

import sys,string
import argparse
import numpy as np
import matplotlib.pyplot as plt
from math import log10,log

class lightcurve() :
    def __init__(self) :
        self.date=[]
        self.flux=[]
        self.err=[]
        self.zp=[]
        self.filter="None"
        self.magsys="None"
        

class sndata() :
    
    def __init__(self,ifilename=None) :
        self.infos={}
        self.data={}
        self.lightcurves={}
        
        self.infos["file"]=ifilename
        if self.infos["file"] != None :
            self.read(self.infos["file"])
        

    def read(self,filename) :
        print "INFO reading",filename
        self.infos["file"]=filename
        file=open(filename)

        keys=[]

        for line in file.readlines() :
            
            if len(line)==0 :
                continue
            
            if line[0]=="@" :
                ivals=line.strip().split(" ")
                key=ivals[0].strip().replace("@","")
                vals=[]
                for val in ivals[1:] :
                    val=val.strip()
                    if len(val)>0 :
                        vals.append(val)                
                self.infos[key]=vals
                continue
            
            if line[0]=="#" :
                if line.find(":")<0 :
                    #print "this is a comment"
                    #print line
                    continue
                if line.find("end")>=0 :
                    #print "this is the end"
                    #print line
                    continue
                ivals=line.strip().replace("#","").split(" ")
                for val in ivals :
                    val=val.replace(":","").strip()
                    if len(val)>0 :
                        self.data[val]=[]
                        keys.append(val) # to keep original ordering
                        break
                continue
            
            ivals=line.strip().split(" ")
            vals=[]
            for val in ivals :
                val=val.strip()
                if len(val)>0 :
                    vals.append(val)
            
            if len(vals)==0 :
                continue
            if len(vals) != len(self.data) :
                print "WARNING ignore the following line, number of data and keys don't match",len(vals),len(keys)
                print line 
                continue
            for k,val in zip(keys,vals) :
                self.data[k].append(val)

        
        # print "===================="
        # print self.infos
        #  print self.data.keys()
        #  print ""
        
        
        # now parse this into light curves
        # one lightcurve per filter and magsys
        
        for i in range(len(self.data["Date"])) :
            if self.data.has_key("Filter") :
                # new format
                filter=self.data["Filter"][i]
                magsys=self.data["MagSys"][i]
            else :
                # old format
                filter=self.infos["INSTRUMENT"][0]+"::"+self.infos["BAND"][0]
                magsys=self.infos["MAGSYS"][0]
                
            identifier=filter+"-"+magsys
            if self.lightcurves.has_key(identifier) :
                lc=self.lightcurves[identifier]
            else :
                lc=lightcurve()
                lc.filter=filter
                lc.magsys=magsys
                self.lightcurves[identifier]=lc
            lc.date.append(string.atof(self.data["Date"][i]))
            lc.flux.append(string.atof(self.data["Flux"][i]))
            lc.err.append(string.atof(self.data["Fluxerr"][i]))
            lc.zp.append(string.atof(self.data["ZP"][i]))

def mags(lc,maxerr=2.) :
    date=[]
    mag=[]
    magerr=[]
    for d,f,e,z in zip(lc.date,lc.flux,lc.err,lc.zp) :
        if f<=0 :
            continue
        
        err=2.5/log(10.)*e/f
        
        if err < maxerr :
            date.append(d)
            mag.append(-2.5*log10(f)+z)
            magerr.append(err)
    return date,mag,magerr

parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
parser.add_argument('--data', help = 'input lightcurve(s) ',nargs='+')
parser.add_argument('--model', help = 'output lightcurve(s) from snlc',nargs='+')
parser.add_argument('--mag', help = 'plot as magnitudes', action = 'store_true')
parser.add_argument('--offset', help = 'apply offset for magnitudes', type=float,default=2.)
args = parser.parse_args()


data=None
model=None

if args.data == None :
    print "need data"
    parser.print_help()
    sys.exit(0)

if len( args.data ) > 0 :
    data=sndata(args.data[0])
    for e in args.data[1:] :
        data.read(e)

if args.model != None :
    if len( args.model ) > 0 :
        model=sndata(args.model[0])
        for e in args.model[1:] :
            model.read(e)

colors=plt.cm.rainbow(np.linspace(0,1,len(data.lightcurves)))

if not args.mag :
    counter=-1
    for identifier,lc in data.lightcurves.iteritems() :
        counter+=1
        color=colors[counter]
        date=np.array(lc.date)
        flux=np.array(lc.flux)
        err=np.array(lc.err)
        plt.errorbar(date,flux,err,fmt="o",label=lc.filter+" ("+lc.magsys+")",color=color)
        if model.lightcurves.has_key(identifier) :
            mlc=model.lightcurves[identifier]
            date=np.array(mlc.date)
            flux=np.array(mlc.flux)
            err=np.array(mlc.err)
            plt.fill_between(date,flux-err,flux+err,color=color,alpha=0.3)
            plt.plot(date,flux,color=color)
    plt.xlabel("date [days]")
    plt.ylabel("flux [arb. units]")

else :
    counter=-1
    
    for identifier,lc in data.lightcurves.iteritems() :
        counter+=1
        color=colors[counter]
        date,mag,magerr=mags(lc)
        date=np.array(date)       
        mag=np.array(mag)+args.offset*counter
        magerr=np.array(magerr)
        label=lc.filter+"("+lc.magsys+")+%2.1f"%(args.offset*counter)
        plt.errorbar(date,mag,magerr,fmt="o",label=label,color=color)
        if model.lightcurves.has_key(identifier) :
            mlc=model.lightcurves[identifier]
            date,mag,magerr=mags(mlc)
            date=np.array(date)           
            mag=np.array(mag)+args.offset*counter
            magerr=np.array(magerr)
            plt.fill_between(date,mag-magerr,mag+magerr,color=color,alpha=0.3)
            plt.plot(date,mag,color=color)
    plt.xlabel("date [days]")
    plt.ylabel("mag")
    
name=""
if data.infos.has_key("SN") :
    name=data.infos["SN"][0]

plt.legend(title=name,loc="upper right",numpoints=1)

if args.mag :
    plt.gca().invert_yaxis()

plt.show()
